 	  CPU	Z80
          org	0b000h 			; b000h Entrypoint MZ7xx 
	  jp	begin			; jump around definitions

; written by KarlHeinz.Mau@t-online.de 1996, last modified may 12th, 1998
;
; This program prints or plots a hardcopy of the screen from Sharp MZ7xx
; computers. 
;
; The type of plotter supported by this program is Sharp MZ1-P01, that was 
; built-in in MZ731 and MZ821 (a long time ago, isn't it?).
;
; The printer used for printing at my site is NEC Pinwriter P6 plus. I can't 
; warrant the correct function with other printers because of the timing of
; the handshaking between my program and your printer. Test it and post me
; your practical experiences, please.
;
; Technical connection:
; You have to connect pin 1 (RDP) of the printer connector of the MZ to
; pin 1 (strobe) to your printer, MZ-pin 19 (IRT) with pin 16 (reset), MZ-pin
; 21 (RDA) with pin 11 (busy), MZ-pin 23 (STA) with pin 10 (ACK) and data
; RD1 to RD8 at MZ-pins 3, 5, 7, 9, 11, 13, 15, 17 to pins 2, 3, 4, 5, 6, 7,
; 8 and 9 and last MZ-pin 25 (GND) to pins 19 to 29.
;
; Program usage: 
; You can call this program in programs written by yourself. In this case
; you have to do the following:
;
; First you must specify if the hardcopy output should be printed or plotted: 
;
;	  ld	a,'P'			; hardcopy for MZ731-plotter or
; 					; any other value for printer
;	  ld	(prtsw),a		; put it into my program area
;
; Then put the following coding into your program to call my program: 
;
;	  ld	a,0ffh			; any non-zeroe value
;	  ld	(retsw),a		; means return to your program
;	  call	0b000h			; execute this program
;
; My program sets the zero flag on return to your program. If the zero flag
; is set, the user has pressed the SHIFT- and BRK-keyes to stop the output.
; In this case you can interrupt your own program or do any other functions.
; For example:
;
;	  ld	a,'I'			; print hardcopy (no 'P')
;	  ld	(prtsw),a
;	  ld	(retsw),a		; non zero value to return to yours 
;	  call	0b000h			; exec hardcopy
;	  jp	z,break			; if brk key go to label break
;	  ...	...			; if not ....
;
; If you want to use this program by the monitor ROM, you must type in into
; monitor's line of prompt:
;
;    JB000
;
; and you have to be sure, that my variable 'retsw' is set to zero and the
; variable 'prtsw' is set to the appropiate value needed (as described
; above) before each execution. If this program will be executed, it returns
; to the monitor program. You can set the variables with monitor's
; 'M'-command:
; 
; MB003 to set variable prtsw and
; MB004 to set variable retsw.
; 
; 
; 
;	constants, definitions

lf:	  equ	0ah			; line feed + autom. cr for printer
cr:	  equ   0dh			; Carriage Return
ST1:	  equ	00adh			; entry of Monitor-ROM
dacn10:	  equ	0bd4h			; conversion routine
screen:	  equ	0d000h			; start of screen area
porta:	  equ	0e000h			; 8255 Port A (write keyboard data)
portb:	  equ	0e001h			; 8255 Port B (read keyboard data)
PrtSw:	  db	'P'			; 'P'=Plot, any other value=print
retsw:	  db	00h			; 00h=go back to monitor,
					; any other value=goback to caller
stackptr: dw	0000h			; stack on entry to hardcopy routine
colors:	  DB	17h			; work colors


;	hardcopy entry

begin:	  push	bc			; save caller's regs
	  push	de
	  push	hl
	  ld	(stackptr),sp		; for futher use at end
begin010: IN	A,(0FEh)
	  BIT	0,A			; busy?
	  JR	Z,begin020		; not busy
	  OR	40h			; set IRT
	  OUT	(0FEh),A		; reset printer
	  CALL	brktst			; test break key
	  JR	NZ,begin010		; no, loop
	  jp	break010		; yes, goback

begin020: LD	HL,screen		; address to screen area
	  PUSH	HL			; save for futher use
begin030: POP	HL			; addr. of current row in screen
	  PUSH	HL			; back for further use
	  LD	B,40			; init loop counter (40 columns/row) 
	  LD	D,00h			
	  LD	E,B			; DE is displacement to next row
	  ADD	HL,DE			; HL is address to next row on screen
	  LD	A,0D4h			; check for last row+1 (starts at
	  CP	H			; address 0d410h)
	  JR	NZ,findsp		; not last
	  LD	A,10h
	  CP	L
	  JR	nZ,findsp		; not last
	  and	a			; accu is 10h, 'and a' sets NZ=no brk
	  JR	break010		; goback

findsp:	  POP	DE			; exchange entries on stack
	  PUSH	HL			; contains address of next row
	  PUSH	DE			; contains address of current row
findsp10: DEC	HL			; skip backwards subsequent spaces
	  LD	A,(HL)			; get char
	  OR	A			; space?
	  JR	NZ,lineout		; no, start outputting, notice,
;					  b-reg contains # of char to output
	  DJNZ	findsp10		; yes,skip space
	  POP	HL			; full space line! rebuild stack
	  JR	begin030		; process next row

lineout:  POP	HL			; curr. column address in curr. row
lout010:  SET	3,H			; address to screen colors, above d800 
	  LD	A,(hl)			; get colors
	  ld	(colors),a
	  rlca				; invert colors
	  rlca
	  rlca
	  rlca
	  LD	(HL),A			; set to invert colors
	  RES	3,H			; address back to the char. screen
	  LD	A,(HL)			; get char to output
	  CALL	dspasc			; convert displ. code to ascii
	  OR	A			; 00h?
	  JR	NZ,lout020		; no
	  OR	20h			; yes, force space on output
lout020:  CALL	outch			; output 1 char
	  jp	z,break			; brk condition occured
	  SET	3,H			; address to screen colors
	  LD	A,(colors)		; get old colors
	  LD	(HL),A			; reset to old colors
	  RES	3,H			; address back to the char. screen
	  INC	HL			; next char
	  DJNZ	lout010			; loop if length(contents of row)>0
	  call	newline			; set pen to next line
	  jp	z,break010		; brk condition occured
	  JR	begin030		; loop for next row

;	skip to new line (for both, printer/plotter)

newline:  ld	a,(prtsw)		; 
	  cp	'P'			; plotter?
	  ld	a,cr			; cr for plotter
	  jr 	z,newl010		; yes, do cr
	  ld	a,lf			; no, lf for printer
newl010:  call	outch			; output cr or lf
	  ret

;  test on brk key

brktst:	  CALL	brktst20		; test brk key
	  RET	NZ			; no, continue
brktst10: CALL	brktst20		; test again and
	  JR	Z,brktst10		; wait until unpressed
	  xor	a			; zero flag indicates break
	  ret

;	test shift-break

brktst20: LD	A,0F8h			; column brk/ctrl/shift
	  LD	(porta),A		; Port A 8255 send keyboard f8h
	  NOP				; wait 1 nop
	  LD	A,(portb)		; Port B 8255 receive keyboard data
	  CP	7Eh			; shift + brk ?
	  RET				; goback

;  goes back to monitor, if brk key used and started from monitor
;			 else returns if brk key used and called by program

break:	  SET	3,H			; address to screen colors 
	  LD	A,(colors)		; get old colors to normalize
	  LD	(HL),A			; may be inverted at this time
	  xor	a			; set zflag (brk condition)
break010: push	af			; save zero flag (z=brk, nz=no brk)
	  LD	A,(retsw)		; 0 means jumped into program by mon.
	  AND	A			; >0 means called by any program
	  JP	z,intrpt		; goback to monitor
	  XOR	A			; prepare next execution parameters
	  LD	(retsw),A
	  pop	af			; notice zflag to caller
	  ld	sp,(stackptr)
	  pop	hl			; restore caller's reg's
	  POP	DE
	  POP	BC
	  ret				; goback to caller

;	plot 1 character

plotch:	  PUSH	BC			; save callers reg
	  LD	C,00h			; 00h=ready
	  LD	B,A			; save
	  CALL	wtack			; wait on ready
	  jr	z,plotch10		; stop on brk
	  LD	A,B			; restore char to plot
	  OUT	(0FFh),A		; data to plotter
	  LD	A,80h			; RDP (strobe) high
	  OUT	(0FEh),A		; to plotter
	  LD	C,01h			; RDA test 
	  CALL	wtack			; wait (ack)
	  jr	z,plotch10		; stop on brk
	  XOR	A
	  OUT	(0FEh),A		; RDP low
	  cpl
	  and	a			; NZ=no brk
plotch10: POP	BC			; restore callers reg
	  RET     			; goback to caller

;  wait for acknowledge (plotter)

wtack:	  IN	A,(0FEh)
	  AND	00Dh			; RDA only
	  CP	C			; test busy
	  jr	nZ,wtack020		; busy, go test brk condition
wtack010: cpl				; ffh/feh
	  and	a			; indicate no break (NZ)
	  ret

wtack020: call	brktst			; test break key
	  JR	nz,wtack		; wait on ready
	  ret				; zflag=brk condition

;	if (prtsw) = "P" outputs 1 character to plotter 
;	else to printer

outch:	  LD	C,A			; save char to output	
	  LD	A,(prtsw)		; get user's input char
	  CP	'P'			; "P"?
	  LD	A,C			; get back char to output
	  JR	Z,plotch		; yes, go plot 1 char
outch10:  IN	A,(0FEh)		; get state of printer
	  BIT	0,A			; busy?
	  JR	Z,printch		; no, continue
	  CALL	brktst			; yes, busy. brk key?
	  JR	NZ,outch10		; no. Wait on ready
	  Ret				; goback with brk condition

;	print 1 char

printch:  LD	A,C			; get back char to print
	  OUT	(0FFh),A		; data out
	  IN	A,(0FEh)		; get state of printer
	  AND	7Fh			; set strobe
	  OUT	(0FEh),A		; put strobe
	  IN	A,(0FEh)		; get state again
	  OR	80h			; reset strobe
	  OUT	(0FEh),A		; put out
prtch10:  IN	A,(0FEh)		; get state
	  BIT	0,A			; test busy
	  jr	Z,wtack010		; not busy, goback to caller
	  CALL	brktst			; brk key?
	  JR	NZ,prtch10		; no, wait on ready (printed)
	  Ret				; yes, goback with brk cond.

;	display to ascii for printer or plotter

dspasc:	  PUSH	AF			; save char to convert
	  LD	A,(prtsw)		; users input char
	  CP	'P'			; "P" (plotter)?
	  JR	NZ,convprt		; no, printer
	  POP	AF			; char to convert
	  CALL	convplot		; display to ascii (plotter)
chgf0:	  CP	0F0h			; display code f0h? (space)
	  RET	NZ			; no
	  AND	20h			; yes, put space
	  RET				; goback
     
;	display to ascii conversion for plotter
;	returns from monitor to caller

convplot: PUSH	BC			; save own regs
	  PUSH	HL
	  PUSH	DE
	  LD	HL,plttrtab		; my translation table for plotter
	  JP	dacn10			; use monitor's conversion routine

;	diplay to ascii conversion for printer

convprt:  POP	AF			; get char to convert
	  PUSH	BC			; save callers reg
	  PUSH	HL
	  PUSH	DE
	  LD	HL,prttrtab		; table display to ascii (printer)
	  LD	C,A
	  LD	B,00h			; bc is displacement into table
	  ADD	HL,BC			; address to char
	  LD	A,(HL)			; get translation char
	  CP	0F0h			; display code for space?
	  JR	NZ,conprt10		; no, continue
	  AND	20h			; yes, translate to 20h
conprt10: POP	DE			; restore callers regs
	  POP	HL
	  POP	BC
	  Jr	chgf0			; if 0f0h conv. to spc, then goback

;	end of program, goback to monitor

intrpt:	  ld	sp,10f0h		; init stackpointer
	  jp	st1			; goback to Monitor-ROM



;	plotter translation table

plttrtab: db    19 dup 00h
	  db	0c3h,0c4h
	  db    12 dup 00h
	  db	61h,62h,63h,64h,65h,66h,67h,68h,69h
	  db	6bh,6ah,2fh,2ah,2eh,2dh
	  db	20h,21h,22h,23h,24h,25h,26h,27h,28h,29h
	  db	4fh,2ch,51h,2bh,57h,49h,55h
	  db	01h,02h,03h,04h,05h,06h,07h,08h,09h
	  db	0ah,0bh,0ch,0dh,0eh,0fh
	  db	10h,11h,12h,13h,14h,15h,16h,17h,18h,19h
	  db	1ah,52h,59h,54h,50h,45h
	  db	27 dup 00h
	  db	0dbh
	  db	4 dup 00h
	  db	40h
	  db	10 dup 00h
	  db	0beh
	  db	6 dup 00h
	  db	85h,0a4h,0a5h,00h,94h,87h,88h,00h,82h,98h,84h,92h
	  db	90h,83h,91h,81h,9ah,97h,93h,95h,89h,00h,0afh,8bh
	  db	86h,96h,00h,0abh,0aah,8ah,8eh,00h,0adh,8dh
	  db	3 dup 00h
	  db	8fh,8ch,0aeh,0ach,9bh,00h,99h,0bch
	  db	7 dup 00h
	  db	5ah
	  db	8 dup 00h
	  db	3eh,7 dup 00h,36h,35 dup 00h,1bh,58h
	  db	00h,00h,60h

;	printer translation table 

prttrtab: db	" ABCDEFGHIJKLMNOPQRSTUVWXYZd"
	  db	4 dup 0f0h
	  db	"0123456789-=;/.,"
	  db	16 dup 0f0h
	  db	7dh,06h,0f0h,0f0h,04h,0f0h,05h,0f0h,0f0h,"?"
	  db	5 dup 00h,":",0f0h,"<[",03h,"]@",0f0h,">",0f0h
	  db	5ch,6 dup 0f0h,0e3h,"!",22h,"#$%&'()+*"
	  db	20 dup 0f0h,0b3h
	  db	61h,62h,63h,64h,65h,66h,67h,68h,69h
	  db	6ah,6bh,6ch,6dh,6eh,6fh,70h,71h,72h,73h
	  db	74h,75h,76h,77h,78h,79h,7ah,84h
	  db	8 dup 0f0h,60h,0f7h,4 dup 0f0h,0e1h,81h
	  db	94h,9ah,8eh,99h,12 dup 0f0h,7bh,0f0h,5eh
	  db	5fh,64 dup 0f0h
